from .base_strategy import BaseStrategy
from .simple_moving_average_strategy import SimpleMovingAverageStrategy
from .update_simple_moving_average_strategy import UpdateSimpleMovingAverageStrategy
from .stochastic_oscillator_strategy import StochasticOscillatorStrategy
from .one_moving_average_strategy import OneMovingAverageStrategy
from .traditional_boll_strategy import TraditionalBollStrategy
from .moving_average_convergence_divergence_strategy import MovingAverageConvergenceDivergenceStrategy
from .price_deviation_strategy import PriceDeviationStrategy

STRATEGIES = {
    'SMA': SimpleMovingAverageStrategy,
    # 'USMA': UpdateSimpleMovingAverageStrategy,
#     'SO': StochasticOscillatorStrategy,
    # 'OMA': OneMovingAverageStrategy,
#     'TB': TraditionalBollStrategy,
#     'MACD': MovingAverageConvergenceDivergenceStrategy
}




from event_trader.strategies import BaseStrategy, OneMovingAverageStrategy, STRATEGIES
from china_stock_data import StockData
from event_trader.utils import get_first_line
import pandas as pd

class StockInfo:
    def __init__(self, symbol: str, stock_kwargs = {}, strategies = None):
        self.symbol = symbol
        self.stock_data = StockData(symbol, **stock_kwargs)
        self.strategies: dict[str, BaseStrategy] = {}
        strategies = strategies if strategies is not None else STRATEGIES
        for key, item in strategies.items():
            self.strategies[key] = item(self.stock_data)
            
    def __getattr__(self, key: str):
        if key in self.strategies:
            return self.strategies[key]
        raise AttributeError(f"'StockInfo' object has no attribute '{key}'")
    
    
    def show(self, **kwargs):
        for key, item in self.strategies.items():
            item.show(**kwargs)
            
    def get_result(self, opt_kwargs = {}):
        arr = []
        for key, item in self.strategies.items():
            item.optimize_parameters(**opt_kwargs)
            arr.append({
                "name": key,
                "description": get_first_line(item.__doc__),
                "parameters": item.parameters,
                'status': item.status(),
                'profit': item.account.get_profit(),
                'data': item.get_factors()
            })
        return pd.DataFrame(arr)

        
        
   


import pandas as pd
from .base_strategy import BaseStrategy
from china_stock_data import StockData
import mplfinance as mpf
from event_trader.config import PRICE_COL


DEFAULT_PARAMS = {
    'window': 5
}

DEFAULT_PARAMS_RANGE = {
    'window': (5, 80)
}

class OneMovingAverageStrategy(BaseStrategy):
    """
    计算一个周期的移动平均值。
    当价格高于移动平均线时，买入；
    当价格低于移动平均线时，卖出。
    """
    def __init__(self, stock_data: StockData, params = None, params_range = None):
        _params = params if params is not None else DEFAULT_PARAMS
        _params_range = params_range if params_range is not None else DEFAULT_PARAMS_RANGE
        super().__init__(stock_data, 'one_moving_average', _params, _params_range, None, ['moving_avg'])
        
    def calculate_factors(self):
        window = self.parameters['window']
        self.data['moving_avg'] = self.data[PRICE_COL].rolling(window=window).mean()
        short_mavg_derivative = self.data['moving_avg'].diff()
        self.data['mavg_derivative'] = short_mavg_derivative.fillna(0)
        
    def should_buy(self, row):
        last1 = self.data.iloc[-2]
        last2 = self.data.iloc[-3]
        last3 = self.data.iloc[-4]
        return row['mavg_derivative'] > 0 and last1['mavg_derivative'] >= 0 and last2['mavg_derivative'] <= 0 and last3['mavg_derivative'] < 0
    
    def should_sell(self, row):
        last1 = self.data.iloc[-2]
        last2 = self.data.iloc[-3]
        last3 = self.data.iloc[-4]
        return row['mavg_derivative'] < 0 and last1['mavg_derivative'] <= 0 and last2['mavg_derivative'] >= 0 and last3['mavg_derivative'] > 0

    def buy_signal(self, row, i) -> bool:
        if i < self.window + 3 or pd.isna(row['moving_avg']):
            return False
        last1 = self.data.iloc[i-1]
        last2 = self.data.iloc[i-2]
        last3 = self.data.iloc[i-3]
        
        return row['mavg_derivative'] > 0 and last1['mavg_derivative'] > 0 and last2['mavg_derivative'] < 0 and last3['mavg_derivative'] < 0

    def sell_signal(self, row, i) -> bool:
        if i < self.window + 3 or pd.isna(row['moving_avg']):
            return False
        last1 = self.data.iloc[i-1]
        last2 = self.data.iloc[i-2]
        last3 = self.data.iloc[i-3]
        return row['mavg_derivative'] < 0 and last1['mavg_derivative'] < 0 and last2['mavg_derivative'] > 0 and last3['mavg_derivative'] > 0
    
    def get_plots(self, data):
        return [
            mpf.make_addplot(data['moving_avg'], width=0.8, color='blue', label=f'{self.parameters["window"]}-Day MA')
        ]   



from event_trader.stock_info import StockInfo
from china_stock_data import StockMarket
import pandas as pd
from concurrent.futures import ThreadPoolExecutor, as_completed

class StocksManager:
    def __init__(self, symbols = None, index = None, start = None, limit = None, stock_kwargs = {}):
        self.stocks: dict[str, StockInfo] = {}
        if symbols is not None:
            self.symbols = symbols
            for symbol in symbols:
                self.stocks[symbol] = StockInfo(symbol)
        if index:
            self.stock_market = StockMarket(index)
            codes = self.stock_market['index_codes']

            # 当 start 或 limit 为 None 时的处理
            if start is None:
                start = 0
            if limit is None:
                limit = len(codes)

            # 确保 limit 不超过列表的长度
            actual_limit = min(start + limit, len(codes))

            for i in range(start, actual_limit):
                symbol = codes[i]
                if symbol not in self.stocks:
                    self.stocks[symbol] = StockInfo(symbol, **stock_kwargs)
        
        
    def get_stock_info(self, symbol):
        if symbol in self.stocks:
            return self.stocks[symbol]  
        return None
    
    def show(self, symbols = None, **kwargs):
        for symbol, stock in self.stocks.items():
            if symbols is not None and symbol not in symbols:
                continue
            stock.show(**kwargs)
    
    def get_result(self, opt_kwargs = {}):
        dataframes = []

        # 定义一个函数来获取结果并添加符号
        def fetch_result(symbol, item):
            df = item.get_result(opt_kwargs = opt_kwargs)
            df['symbol'] = symbol
            return df

        # 使用ThreadPoolExecutor来管理线程池
        with ThreadPoolExecutor(max_workers=5) as executor:
            # 提交所有任务并收集future对象
            futures = {executor.submit(fetch_result, symbol, item): symbol for symbol, item in self.stocks.items()}

            # 处理完成的任务
            for future in as_completed(futures):
                symbol = futures[future]
                try:
                    df = future.result()
                    dataframes.append(df)
                except Exception as e:
                    print(f"Error processing {symbol}: {e}")

        if dataframes:
            result_df = pd.concat(dataframes, ignore_index=True)
            return result_df
        else:
            print("No data to combine.")
            return pd.DataFrame()



import pandas as pd
from .simple_moving_average_strategy import SimpleMovingAverageStrategy
from china_stock_data import StockData
import mplfinance as mpf
from event_trader.config import PRICE_COL

DEFAULT_PARAMS = {
    'short_window': 5,
    'long_window': 20,
    'sell_window': 5,
}

DEFAULT_PARAMS_RANGE = {
    'short_window': (2, 20),
    'long_window': (3, 80),
    'sell_window': (3, 30),
}

class UpdateSimpleMovingAverageStrategy(SimpleMovingAverageStrategy):
    """
    升级版的移动平均指标。
    """
    def __init__(self, stock_data: StockData, params = None, params_range=None):
        _params = params if params is not None else DEFAULT_PARAMS
        _params_range = params_range if params_range is not None else DEFAULT_PARAMS_RANGE
        super().__init__(
            stock_data, 
            sub_path='update_simple_moving_average', 
            params=_params, 
            params_range=_params_range, 
            factors=['short_mavg', 'long_mavg', 'sell_mavg']
        )

    def calculate_factors(self):
        super().calculate_factors()
        self.data['sell_mavg'] = self.data[PRICE_COL].rolling(window=self.sell_window).mean()
    
    def should_sell(self, row):
        return row[PRICE_COL] < row['sell_mavg']

    def sell_signal(self, row, i) -> bool:
        if i == 0 or pd.isna(row['sell_mavg']):
            return False
        return self.should_sell(row)
        
    def get_plots(self, data):
        return [
            mpf.make_addplot(data['short_mavg'], width=0.8, color='blue', label=f'{self.short_window}-Day MA'),
            mpf.make_addplot(data['long_mavg'], width=0.8, color='orange', label=f'{self.long_window}-Day MA'),
            mpf.make_addplot(data['sell_mavg'], width=0.8, color='red', label=f'{self.sell_window}-Day MA')
        ]


